'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_4_2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_4_2',error_checker=_errors._error_checker)
GL_ACTIVE_ATOMIC_COUNTER_BUFFERS=_C('GL_ACTIVE_ATOMIC_COUNTER_BUFFERS',0x92D9)
GL_ALL_BARRIER_BITS=_C('GL_ALL_BARRIER_BITS',0xFFFFFFFF)
GL_ATOMIC_COUNTER_BARRIER_BIT=_C('GL_ATOMIC_COUNTER_BARRIER_BIT',0x00001000)
GL_ATOMIC_COUNTER_BUFFER=_C('GL_ATOMIC_COUNTER_BUFFER',0x92C0)
GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTERS=_C('GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTERS',0x92C5)
GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTER_INDICES=_C('GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTER_INDICES',0x92C6)
GL_ATOMIC_COUNTER_BUFFER_BINDING=_C('GL_ATOMIC_COUNTER_BUFFER_BINDING',0x92C1)
GL_ATOMIC_COUNTER_BUFFER_DATA_SIZE=_C('GL_ATOMIC_COUNTER_BUFFER_DATA_SIZE',0x92C4)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_FRAGMENT_SHADER=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_FRAGMENT_SHADER',0x92CB)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_GEOMETRY_SHADER=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_GEOMETRY_SHADER',0x92CA)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_CONTROL_SHADER=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_CONTROL_SHADER',0x92C8)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_EVALUATION_SHADER=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_EVALUATION_SHADER',0x92C9)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_VERTEX_SHADER=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_VERTEX_SHADER',0x92C7)
GL_ATOMIC_COUNTER_BUFFER_SIZE=_C('GL_ATOMIC_COUNTER_BUFFER_SIZE',0x92C3)
GL_ATOMIC_COUNTER_BUFFER_START=_C('GL_ATOMIC_COUNTER_BUFFER_START',0x92C2)
GL_BUFFER_UPDATE_BARRIER_BIT=_C('GL_BUFFER_UPDATE_BARRIER_BIT',0x00000200)
GL_COMMAND_BARRIER_BIT=_C('GL_COMMAND_BARRIER_BIT',0x00000040)
GL_COMPRESSED_RGBA_BPTC_UNORM=_C('GL_COMPRESSED_RGBA_BPTC_UNORM',0x8E8C)
GL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT=_C('GL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT',0x8E8E)
GL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT=_C('GL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT',0x8E8F)
GL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM=_C('GL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM',0x8E8D)
GL_COPY_READ_BUFFER_BINDING=_C('GL_COPY_READ_BUFFER_BINDING',0x8F36)
GL_COPY_WRITE_BUFFER_BINDING=_C('GL_COPY_WRITE_BUFFER_BINDING',0x8F37)
GL_ELEMENT_ARRAY_BARRIER_BIT=_C('GL_ELEMENT_ARRAY_BARRIER_BIT',0x00000002)
GL_FRAMEBUFFER_BARRIER_BIT=_C('GL_FRAMEBUFFER_BARRIER_BIT',0x00000400)
GL_IMAGE_1D=_C('GL_IMAGE_1D',0x904C)
GL_IMAGE_1D_ARRAY=_C('GL_IMAGE_1D_ARRAY',0x9052)
GL_IMAGE_2D=_C('GL_IMAGE_2D',0x904D)
GL_IMAGE_2D_ARRAY=_C('GL_IMAGE_2D_ARRAY',0x9053)
GL_IMAGE_2D_MULTISAMPLE=_C('GL_IMAGE_2D_MULTISAMPLE',0x9055)
GL_IMAGE_2D_MULTISAMPLE_ARRAY=_C('GL_IMAGE_2D_MULTISAMPLE_ARRAY',0x9056)
GL_IMAGE_2D_RECT=_C('GL_IMAGE_2D_RECT',0x904F)
GL_IMAGE_3D=_C('GL_IMAGE_3D',0x904E)
GL_IMAGE_BINDING_ACCESS=_C('GL_IMAGE_BINDING_ACCESS',0x8F3E)
GL_IMAGE_BINDING_FORMAT=_C('GL_IMAGE_BINDING_FORMAT',0x906E)
GL_IMAGE_BINDING_LAYER=_C('GL_IMAGE_BINDING_LAYER',0x8F3D)
GL_IMAGE_BINDING_LAYERED=_C('GL_IMAGE_BINDING_LAYERED',0x8F3C)
GL_IMAGE_BINDING_LEVEL=_C('GL_IMAGE_BINDING_LEVEL',0x8F3B)
GL_IMAGE_BINDING_NAME=_C('GL_IMAGE_BINDING_NAME',0x8F3A)
GL_IMAGE_BUFFER=_C('GL_IMAGE_BUFFER',0x9051)
GL_IMAGE_CUBE=_C('GL_IMAGE_CUBE',0x9050)
GL_IMAGE_CUBE_MAP_ARRAY=_C('GL_IMAGE_CUBE_MAP_ARRAY',0x9054)
GL_IMAGE_FORMAT_COMPATIBILITY_BY_CLASS=_C('GL_IMAGE_FORMAT_COMPATIBILITY_BY_CLASS',0x90C9)
GL_IMAGE_FORMAT_COMPATIBILITY_BY_SIZE=_C('GL_IMAGE_FORMAT_COMPATIBILITY_BY_SIZE',0x90C8)
GL_IMAGE_FORMAT_COMPATIBILITY_TYPE=_C('GL_IMAGE_FORMAT_COMPATIBILITY_TYPE',0x90C7)
GL_INT_IMAGE_1D=_C('GL_INT_IMAGE_1D',0x9057)
GL_INT_IMAGE_1D_ARRAY=_C('GL_INT_IMAGE_1D_ARRAY',0x905D)
GL_INT_IMAGE_2D=_C('GL_INT_IMAGE_2D',0x9058)
GL_INT_IMAGE_2D_ARRAY=_C('GL_INT_IMAGE_2D_ARRAY',0x905E)
GL_INT_IMAGE_2D_MULTISAMPLE=_C('GL_INT_IMAGE_2D_MULTISAMPLE',0x9060)
GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY=_C('GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY',0x9061)
GL_INT_IMAGE_2D_RECT=_C('GL_INT_IMAGE_2D_RECT',0x905A)
GL_INT_IMAGE_3D=_C('GL_INT_IMAGE_3D',0x9059)
GL_INT_IMAGE_BUFFER=_C('GL_INT_IMAGE_BUFFER',0x905C)
GL_INT_IMAGE_CUBE=_C('GL_INT_IMAGE_CUBE',0x905B)
GL_INT_IMAGE_CUBE_MAP_ARRAY=_C('GL_INT_IMAGE_CUBE_MAP_ARRAY',0x905F)
GL_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS=_C('GL_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS',0x92DC)
GL_MAX_ATOMIC_COUNTER_BUFFER_SIZE=_C('GL_MAX_ATOMIC_COUNTER_BUFFER_SIZE',0x92D8)
GL_MAX_COMBINED_ATOMIC_COUNTERS=_C('GL_MAX_COMBINED_ATOMIC_COUNTERS',0x92D7)
GL_MAX_COMBINED_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_COMBINED_ATOMIC_COUNTER_BUFFERS',0x92D1)
GL_MAX_COMBINED_IMAGE_UNIFORMS=_C('GL_MAX_COMBINED_IMAGE_UNIFORMS',0x90CF)
GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS=_C('GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS',0x8F39)
GL_MAX_FRAGMENT_ATOMIC_COUNTERS=_C('GL_MAX_FRAGMENT_ATOMIC_COUNTERS',0x92D6)
GL_MAX_FRAGMENT_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_FRAGMENT_ATOMIC_COUNTER_BUFFERS',0x92D0)
GL_MAX_FRAGMENT_IMAGE_UNIFORMS=_C('GL_MAX_FRAGMENT_IMAGE_UNIFORMS',0x90CE)
GL_MAX_GEOMETRY_ATOMIC_COUNTERS=_C('GL_MAX_GEOMETRY_ATOMIC_COUNTERS',0x92D5)
GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS',0x92CF)
GL_MAX_GEOMETRY_IMAGE_UNIFORMS=_C('GL_MAX_GEOMETRY_IMAGE_UNIFORMS',0x90CD)
GL_MAX_IMAGE_SAMPLES=_C('GL_MAX_IMAGE_SAMPLES',0x906D)
GL_MAX_IMAGE_UNITS=_C('GL_MAX_IMAGE_UNITS',0x8F38)
GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS=_C('GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS',0x92D3)
GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS',0x92CD)
GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS=_C('GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS',0x90CB)
GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS=_C('GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS',0x92D4)
GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS',0x92CE)
GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS=_C('GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS',0x90CC)
GL_MAX_VERTEX_ATOMIC_COUNTERS=_C('GL_MAX_VERTEX_ATOMIC_COUNTERS',0x92D2)
GL_MAX_VERTEX_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_VERTEX_ATOMIC_COUNTER_BUFFERS',0x92CC)
GL_MAX_VERTEX_IMAGE_UNIFORMS=_C('GL_MAX_VERTEX_IMAGE_UNIFORMS',0x90CA)
GL_MIN_MAP_BUFFER_ALIGNMENT=_C('GL_MIN_MAP_BUFFER_ALIGNMENT',0x90BC)
GL_NUM_SAMPLE_COUNTS=_C('GL_NUM_SAMPLE_COUNTS',0x9380)
GL_PACK_COMPRESSED_BLOCK_DEPTH=_C('GL_PACK_COMPRESSED_BLOCK_DEPTH',0x912D)
GL_PACK_COMPRESSED_BLOCK_HEIGHT=_C('GL_PACK_COMPRESSED_BLOCK_HEIGHT',0x912C)
GL_PACK_COMPRESSED_BLOCK_SIZE=_C('GL_PACK_COMPRESSED_BLOCK_SIZE',0x912E)
GL_PACK_COMPRESSED_BLOCK_WIDTH=_C('GL_PACK_COMPRESSED_BLOCK_WIDTH',0x912B)
GL_PIXEL_BUFFER_BARRIER_BIT=_C('GL_PIXEL_BUFFER_BARRIER_BIT',0x00000080)
GL_SHADER_IMAGE_ACCESS_BARRIER_BIT=_C('GL_SHADER_IMAGE_ACCESS_BARRIER_BIT',0x00000020)
GL_TEXTURE_FETCH_BARRIER_BIT=_C('GL_TEXTURE_FETCH_BARRIER_BIT',0x00000008)
GL_TEXTURE_IMMUTABLE_FORMAT=_C('GL_TEXTURE_IMMUTABLE_FORMAT',0x912F)
GL_TEXTURE_UPDATE_BARRIER_BIT=_C('GL_TEXTURE_UPDATE_BARRIER_BIT',0x00000100)
GL_TRANSFORM_FEEDBACK_ACTIVE=_C('GL_TRANSFORM_FEEDBACK_ACTIVE',0x8E24)
GL_TRANSFORM_FEEDBACK_BARRIER_BIT=_C('GL_TRANSFORM_FEEDBACK_BARRIER_BIT',0x00000800)
GL_TRANSFORM_FEEDBACK_PAUSED=_C('GL_TRANSFORM_FEEDBACK_PAUSED',0x8E23)
GL_UNIFORM_ATOMIC_COUNTER_BUFFER_INDEX=_C('GL_UNIFORM_ATOMIC_COUNTER_BUFFER_INDEX',0x92DA)
GL_UNIFORM_BARRIER_BIT=_C('GL_UNIFORM_BARRIER_BIT',0x00000004)
GL_UNPACK_COMPRESSED_BLOCK_DEPTH=_C('GL_UNPACK_COMPRESSED_BLOCK_DEPTH',0x9129)
GL_UNPACK_COMPRESSED_BLOCK_HEIGHT=_C('GL_UNPACK_COMPRESSED_BLOCK_HEIGHT',0x9128)
GL_UNPACK_COMPRESSED_BLOCK_SIZE=_C('GL_UNPACK_COMPRESSED_BLOCK_SIZE',0x912A)
GL_UNPACK_COMPRESSED_BLOCK_WIDTH=_C('GL_UNPACK_COMPRESSED_BLOCK_WIDTH',0x9127)
GL_UNSIGNED_INT_ATOMIC_COUNTER=_C('GL_UNSIGNED_INT_ATOMIC_COUNTER',0x92DB)
GL_UNSIGNED_INT_IMAGE_1D=_C('GL_UNSIGNED_INT_IMAGE_1D',0x9062)
GL_UNSIGNED_INT_IMAGE_1D_ARRAY=_C('GL_UNSIGNED_INT_IMAGE_1D_ARRAY',0x9068)
GL_UNSIGNED_INT_IMAGE_2D=_C('GL_UNSIGNED_INT_IMAGE_2D',0x9063)
GL_UNSIGNED_INT_IMAGE_2D_ARRAY=_C('GL_UNSIGNED_INT_IMAGE_2D_ARRAY',0x9069)
GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE=_C('GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE',0x906B)
GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY=_C('GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY',0x906C)
GL_UNSIGNED_INT_IMAGE_2D_RECT=_C('GL_UNSIGNED_INT_IMAGE_2D_RECT',0x9065)
GL_UNSIGNED_INT_IMAGE_3D=_C('GL_UNSIGNED_INT_IMAGE_3D',0x9064)
GL_UNSIGNED_INT_IMAGE_BUFFER=_C('GL_UNSIGNED_INT_IMAGE_BUFFER',0x9067)
GL_UNSIGNED_INT_IMAGE_CUBE=_C('GL_UNSIGNED_INT_IMAGE_CUBE',0x9066)
GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY=_C('GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY',0x906A)
GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT=_C('GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT',0x00000001)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLboolean,_cs.GLint,_cs.GLenum,_cs.GLenum)
def glBindImageTexture(unit,texture,level,layered,layer,access,format):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLuint)
def glDrawArraysInstancedBaseInstance(mode,first,count,instancecount,baseinstance):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei,_cs.GLuint)
def glDrawElementsInstancedBaseInstance(mode,count,type,indices,instancecount,baseinstance):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei,_cs.GLint,_cs.GLuint)
def glDrawElementsInstancedBaseVertexBaseInstance(mode,count,type,indices,instancecount,basevertex,baseinstance):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei)
def glDrawTransformFeedbackInstanced(mode,id,instancecount):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei)
def glDrawTransformFeedbackStreamInstanced(mode,id,stream,instancecount):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetActiveAtomicCounterBufferiv(program,bufferIndex,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLsizei,arrays.GLintArray)
def glGetInternalformativ(target,internalformat,pname,bufSize,params):pass
@_f
@_p.types(None,_cs.GLbitfield)
def glMemoryBarrier(barriers):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei)
def glTexStorage1D(target,levels,internalformat,width):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glTexStorage2D(target,levels,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glTexStorage3D(target,levels,internalformat,width,height,depth):pass



from OpenGL.error import _ErrorChecker
from OpenGL import platform as _p

if _ErrorChecker:
    _error_checker = _ErrorChecker( 
        _p.PLATFORM, 
        _p.PLATFORM.EGL.eglGetError, 
        0x3000 # EGL_SUCCESS
    )
else:
    _ErrorChecker = None



'''OpenGL extension SGIS.pixel_texture

This module customises the behaviour of the 
OpenGL.raw.GL.SGIS.pixel_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	The geometry rasterization and pixel pipeline "convert to fragment"
	stages each produce fragments.  The fragments are processed by
	a unified per fragment pipeline that begins with the application
	of the texture to the fragment color.  Because the pixel pipeline
	shares the per fragment processing with the geometry pipeline, the
	fragments produced by the pixel pipeline must have the same fields
	as the ones produced by the geometry pipeline.  When
	pixel groups are being converted to fragments, the parts 
	of the fragment that aren't derived from the pixel groups 
	are taken from the associated values in the current raster position.
	
	A fragment consists of x and y window coordinates and their
	associated color value, depth value, and texture coordinates.
	In the 1.1 OpenGL specification, when the pixel group is RGBA
	the fragment color is always derived from the pixel group, 
	and the depth value and texture coordinates always come 
	from the raster position.
	
	This extension provides a way to specify how the texture coordinates
	of the fragments can be derived from RGBA pixel groups.  When
	this option is enabled, the source of the fragment color value
	when the pixel group is RGBA can be specified to come from either 
	the raster position or the pixel group.
	
	Deriving the fragment texture coordinates from the pixel group
	effectively converts a color image into a texture coordinate image.
	The multidimensional texture mapping lookup logic also makes this
	extension useful for implementing multidimensional color lookups.
	Multidimensional color lookups can be used to implement very
	accurate color space conversions.
	
	Deriving texture coordinates from the pixel groups in the pixel
	pipeline introduces a problem with the lambda parameter in the
	texture mapping equations.  When texture coordinates are
	being taken from the current raster position texture coordinates,
	the texture coordinate values don't change from pixel to pixel,
	and the equation for calculating lambda always produces zero.
	Enabling pixel_texture introduces changes in the texture
	coordinates from pixel to pixel which are not necessarily
	meaningful for texture lookups.  This problem is addressed
	by specifying that lambda is always set to zero when pixel_texture 
	is enabled.  

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIS/pixel_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIS.pixel_texture import *
from OpenGL.raw.GL.SGIS.pixel_texture import _EXTENSION_NAME

def glInitPixelTextureSGIS():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glPixelTexGenParameterivSGIS.params size not checked against 'pname'
glPixelTexGenParameterivSGIS=wrapper.wrapper(glPixelTexGenParameterivSGIS).setInputArraySize(
    'params', None
)
# INPUT glPixelTexGenParameterfvSGIS.params size not checked against 'pname'
glPixelTexGenParameterfvSGIS=wrapper.wrapper(glPixelTexGenParameterfvSGIS).setInputArraySize(
    'params', None
)
glGetPixelTexGenParameterivSGIS=wrapper.wrapper(glGetPixelTexGenParameterivSGIS).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetPixelTexGenParameterfvSGIS=wrapper.wrapper(glGetPixelTexGenParameterfvSGIS).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_APPLE_texture_packed_float'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_APPLE_texture_packed_float',error_checker=_errors._error_checker)
GL_R11F_G11F_B10F_APPLE=_C('GL_R11F_G11F_B10F_APPLE',0x8C3A)
GL_RGB9_E5_APPLE=_C('GL_RGB9_E5_APPLE',0x8C3D)
GL_UNSIGNED_INT_10F_11F_11F_REV_APPLE=_C('GL_UNSIGNED_INT_10F_11F_11F_REV_APPLE',0x8C3B)
GL_UNSIGNED_INT_5_9_9_9_REV_APPLE=_C('GL_UNSIGNED_INT_5_9_9_9_REV_APPLE',0x8C3E)




'''OpenGL extension ARB.conditional_render_inverted

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.conditional_render_inverted to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds new modes to BeginConditionalRender which invert
	the condition used to determine whether to draw or not.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/conditional_render_inverted.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.conditional_render_inverted import *
from OpenGL.raw.GL.ARB.conditional_render_inverted import _EXTENSION_NAME

def glInitConditionalRenderInvertedARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_gpu_program4'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_gpu_program4',error_checker=_errors._error_checker)
GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV=_C('GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV',0x8908)
GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV=_C('GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV',0x8DA5)
GL_MAX_PROGRAM_GENERIC_RESULTS_NV=_C('GL_MAX_PROGRAM_GENERIC_RESULTS_NV',0x8DA6)
GL_MAX_PROGRAM_RESULT_COMPONENTS_NV=_C('GL_MAX_PROGRAM_RESULT_COMPONENTS_NV',0x8909)
GL_MAX_PROGRAM_TEXEL_OFFSET_NV=_C('GL_MAX_PROGRAM_TEXEL_OFFSET_NV',0x8905)
GL_MIN_PROGRAM_TEXEL_OFFSET_NV=_C('GL_MIN_PROGRAM_TEXEL_OFFSET_NV',0x8904)
GL_PROGRAM_ATTRIB_COMPONENTS_NV=_C('GL_PROGRAM_ATTRIB_COMPONENTS_NV',0x8906)
GL_PROGRAM_RESULT_COMPONENTS_NV=_C('GL_PROGRAM_RESULT_COMPONENTS_NV',0x8907)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glGetProgramEnvParameterIivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLuintArray)
def glGetProgramEnvParameterIuivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glGetProgramLocalParameterIivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLuintArray)
def glGetProgramLocalParameterIuivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glProgramEnvParameterI4iNV(target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glProgramEnvParameterI4ivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glProgramEnvParameterI4uiNV(target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLuintArray)
def glProgramEnvParameterI4uivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLintArray)
def glProgramEnvParametersI4ivNV(target,index,count,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray)
def glProgramEnvParametersI4uivNV(target,index,count,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glProgramLocalParameterI4iNV(target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glProgramLocalParameterI4ivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glProgramLocalParameterI4uiNV(target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLuintArray)
def glProgramLocalParameterI4uivNV(target,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLintArray)
def glProgramLocalParametersI4ivNV(target,index,count,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray)
def glProgramLocalParametersI4uivNV(target,index,count,params):pass



'''OpenGL extension ARB.transform_feedback_overflow_query

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.transform_feedback_overflow_query to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds new query types which can be used to detect overflow
	of transform feedback buffers. The new query types are also accepted by
	conditional rendering commands.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/transform_feedback_overflow_query.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.transform_feedback_overflow_query import *
from OpenGL.raw.GL.ARB.transform_feedback_overflow_query import _EXTENSION_NAME

def glInitTransformFeedbackOverflowQueryARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_compute_shader'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_compute_shader',error_checker=_errors._error_checker)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_COMPUTE_SHADER=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_COMPUTE_SHADER',0x90ED)
GL_COMPUTE_SHADER=_C('GL_COMPUTE_SHADER',0x91B9)
GL_COMPUTE_SHADER_BIT=_C('GL_COMPUTE_SHADER_BIT',0x00000020)
GL_COMPUTE_WORK_GROUP_SIZE=_C('GL_COMPUTE_WORK_GROUP_SIZE',0x8267)
GL_DISPATCH_INDIRECT_BUFFER=_C('GL_DISPATCH_INDIRECT_BUFFER',0x90EE)
GL_DISPATCH_INDIRECT_BUFFER_BINDING=_C('GL_DISPATCH_INDIRECT_BUFFER_BINDING',0x90EF)
GL_MAX_COMBINED_COMPUTE_UNIFORM_COMPONENTS=_C('GL_MAX_COMBINED_COMPUTE_UNIFORM_COMPONENTS',0x8266)
GL_MAX_COMPUTE_ATOMIC_COUNTERS=_C('GL_MAX_COMPUTE_ATOMIC_COUNTERS',0x8265)
GL_MAX_COMPUTE_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_COMPUTE_ATOMIC_COUNTER_BUFFERS',0x8264)
GL_MAX_COMPUTE_IMAGE_UNIFORMS=_C('GL_MAX_COMPUTE_IMAGE_UNIFORMS',0x91BD)
GL_MAX_COMPUTE_SHARED_MEMORY_SIZE=_C('GL_MAX_COMPUTE_SHARED_MEMORY_SIZE',0x8262)
GL_MAX_COMPUTE_TEXTURE_IMAGE_UNITS=_C('GL_MAX_COMPUTE_TEXTURE_IMAGE_UNITS',0x91BC)
GL_MAX_COMPUTE_UNIFORM_BLOCKS=_C('GL_MAX_COMPUTE_UNIFORM_BLOCKS',0x91BB)
GL_MAX_COMPUTE_UNIFORM_COMPONENTS=_C('GL_MAX_COMPUTE_UNIFORM_COMPONENTS',0x8263)
GL_MAX_COMPUTE_WORK_GROUP_COUNT=_C('GL_MAX_COMPUTE_WORK_GROUP_COUNT',0x91BE)
GL_MAX_COMPUTE_WORK_GROUP_INVOCATIONS=_C('GL_MAX_COMPUTE_WORK_GROUP_INVOCATIONS',0x90EB)
GL_MAX_COMPUTE_WORK_GROUP_SIZE=_C('GL_MAX_COMPUTE_WORK_GROUP_SIZE',0x91BF)
GL_UNIFORM_BLOCK_REFERENCED_BY_COMPUTE_SHADER=_C('GL_UNIFORM_BLOCK_REFERENCED_BY_COMPUTE_SHADER',0x90EC)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glDispatchCompute(num_groups_x,num_groups_y,num_groups_z):pass
@_f
@_p.types(None,_cs.GLintptr)
def glDispatchComputeIndirect(indirect):pass



'''OpenGL extension ARB.texture_border_clamp

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_border_clamp to provide a more 
Python-friendly API

Overview (from the spec)
	
	The base OpenGL provides clamping such that the texture coordinates are
	limited to exactly the range [0,1].  When a texture coordinate is clamped
	using this algorithm, the texture sampling filter straddles the edge of
	the texture image, taking 1/2 its sample values from within the texture
	image, and the other 1/2 from the texture border.  It is sometimes
	desirable for a texture to be clamped to the border color, rather than to
	an average of the border and edge colors.
	
	This extension defines an additional texture clamping algorithm.
	CLAMP_TO_BORDER_ARB clamps texture coordinates at all mipmap levels such
	that NEAREST and LINEAR filters return only the color of the border
	texels.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_border_clamp.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_border_clamp import *
from OpenGL.raw.GL.ARB.texture_border_clamp import _EXTENSION_NAME

def glInitTextureBorderClampARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Implementation of GLU Nurbs structure and callback methods

Same basic pattern as seen with the gluTess* functions, just need to
add some bookkeeping to the structure class so that we can keep the
Python function references alive during the calling process.
"""
from OpenGL.raw import GLU as _simple
from OpenGL import platform, converters, wrapper
from OpenGL.GLU import glustruct
from OpenGL.lazywrapper import lazy as _lazy
from OpenGL import arrays, error
import ctypes
import weakref
from OpenGL.platform import PLATFORM
import OpenGL
from OpenGL import _configflags

__all__ = (
    'GLUnurbs',
    'gluNewNurbsRenderer',
    'gluNurbsCallback',
    'gluNurbsCallbackData',
    'gluNurbsCallbackDataEXT',
    'gluNurbsCurve',
    'gluNurbsSurface',
    'gluPwlCurve',
)

# /usr/include/GL/glu.h 242
class GLUnurbs(glustruct.GLUStruct, _simple.GLUnurbs):
    """GLU Nurbs structure with oor and callback storage support

    IMPORTANT NOTE: the texture coordinate callback receives a raw ctypes
    data-pointer, as without knowing what type of evaluation is being done
    (1D or 2D) we cannot safely determine the size of the array to convert
    it.  This is a limitation of the C implementation.  To convert to regular
    data-pointer, just call yourNurb.ptrAsArray( ptr, size, arrays.GLfloatArray )
    with the size of data you expect.
    """
    FUNCTION_TYPE = PLATFORM.functionTypeFor(PLATFORM.GLU)
    CALLBACK_FUNCTION_REGISTRARS = {
        # mapping from "which" to a function that should take 3 parameters,
        # the nurb, the which and the function pointer...
    }
    CALLBACK_TYPES = {
        # mapping from "which" GLU enumeration to a ctypes function type
        _simple.GLU_NURBS_BEGIN: FUNCTION_TYPE(
            None, _simple.GLenum
        ),
        _simple.GLU_NURBS_BEGIN_DATA: FUNCTION_TYPE(
            None, _simple.GLenum, ctypes.POINTER(_simple.GLvoid)
        ),
        _simple.GLU_NURBS_VERTEX: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat)
        ),
        _simple.GLU_NURBS_VERTEX_DATA: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat), ctypes.POINTER(_simple.GLvoid)
        ),
        _simple.GLU_NURBS_NORMAL: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat)
        ),
        _simple.GLU_NURBS_NORMAL_DATA: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat), ctypes.POINTER(_simple.GLvoid)
        ),
        _simple.GLU_NURBS_COLOR: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat)
        ),
        _simple.GLU_NURBS_COLOR_DATA: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat), ctypes.POINTER(_simple.GLvoid)
        ),
        _simple.GLU_NURBS_TEXTURE_COORD: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat)
        ),
        _simple.GLU_NURBS_TEXTURE_COORD_DATA: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLfloat), ctypes.POINTER(_simple.GLvoid)
        ),
        _simple.GLU_NURBS_END:FUNCTION_TYPE(
            None
        ),
        _simple.GLU_NURBS_END_DATA: FUNCTION_TYPE(
            None, ctypes.POINTER(_simple.GLvoid)
        ),
        _simple.GLU_NURBS_ERROR:FUNCTION_TYPE(
            None, _simple.GLenum,
        ),
    }
    WRAPPER_METHODS = {
        _simple.GLU_NURBS_BEGIN: None,
        _simple.GLU_NURBS_BEGIN_DATA: '_justOOR',
        _simple.GLU_NURBS_VERTEX: '_vec3',
        _simple.GLU_NURBS_VERTEX_DATA: '_vec3',
        _simple.GLU_NURBS_NORMAL: '_vec3',
        _simple.GLU_NURBS_NORMAL_DATA: '_vec3',
        _simple.GLU_NURBS_COLOR: '_vec4',
        _simple.GLU_NURBS_COLOR_DATA: '_vec4',
        _simple.GLU_NURBS_TEXTURE_COORD: '_tex',
        _simple.GLU_NURBS_TEXTURE_COORD_DATA: '_tex',
        _simple.GLU_NURBS_END: None,
        _simple.GLU_NURBS_END_DATA: '_justOOR',
        _simple.GLU_NURBS_ERROR: None,
    }
    def _justOOR( self, function ):
        """Just do OOR on the last argument..."""
        def getOOR( *args ):
            args = args[:-1] + (self.originalObject(args[-1]),)
            return function( *args )
        return getOOR
    def _vec3( self, function, size=3 ):
        """Convert first arg to size-element array, do OOR on arg2 if present"""
        def vec( *args ):
            vec = self.ptrAsArray(args[0],size,arrays.GLfloatArray)
            if len(args) > 1:
                oor = self.originalObject(args[1])
                return function( vec, oor )
            else:
                return function( vec )
        return vec
    def _vec4( self, function ):
        """Size-4 vector version..."""
        return self._vec3( function, 4 )
    def _tex( self, function ):
        """Texture coordinate callback

        NOTE: there is no way for *us* to tell what size the array is, you will
        get back a raw data-point, not an array, as you do for all other callback
        types!!!
        """
        def oor( *args ):
            if len(args) > 1:
                oor = self.originalObject(args[1])
                return function( args[0], oor )
            else:
                return function( args[0] )
        return oor

# XXX yes, this is a side-effect...
_simple.gluNewNurbsRenderer.restype = ctypes.POINTER( GLUnurbs )

def _callbackWithType( funcType ):
    """Get gluNurbsCallback function with set last arg-type"""
    result =  platform.copyBaseFunction(
        _simple.gluNurbsCallback
    )
    result.argtypes = [ctypes.POINTER(GLUnurbs), _simple.GLenum, funcType]
    assert result.argtypes[-1] == funcType
    return result

for (c,funcType) in GLUnurbs.CALLBACK_TYPES.items():
    cb = _callbackWithType( funcType )
    GLUnurbs.CALLBACK_FUNCTION_REGISTRARS[ c ] = cb
    assert funcType == GLUnurbs.CALLBACK_TYPES[c]
    assert cb.argtypes[-1] == funcType
try:
    del c,cb, funcType
except NameError as err:
    pass

def gluNurbsCallback( nurb, which, CallBackFunc ):
    """Dispatch to the nurb's addCallback operation"""
    return nurb.addCallback( which, CallBackFunc )

@_lazy( _simple.gluNewNurbsRenderer )
def gluNewNurbsRenderer( baseFunction ):
    """Return a new nurbs renderer for the system (dereferences pointer)"""
    newSet = baseFunction()
    new = newSet[0]
    #new.__class__ = GLUnurbs # yes, I know, ick
    return new

@_lazy( _simple.gluNurbsCallbackData )
def gluNurbsCallbackData( baseFunction, nurb, userData ):
    """Note the Python object for use as userData by the nurb"""
    return baseFunction(
        nurb, nurb.noteObject( userData )
    )

MAX_ORDER = 8
def checkOrder( order,knotCount,name ):
    """Check that order is valid..."""
    if order < 1:
        raise error.GLUError(
            """%s should be 1 or more, is %s"""%( name,order,)
        )
    elif order > MAX_ORDER:
        raise error.GLUError(
            """%s should be %s or less, is %s"""%( name, MAX_ORDER, order)
        )
    elif knotCount < (2*order):
        raise error.GLUError(
            """Knotcount must be at least 2x %s is %s should be at least %s"""%( name, knotCount, 2*order)
        )
def checkKnots( knots, name ):
    """Check that knots are in ascending order"""
    if len(knots):
        knot = knots[0]
        for next in knots[1:]:
            if next < knot:
                raise error.GLUError(
                    """%s has decreasing knot %s after %s"""%( name, next, knot )
                )

@_lazy( _simple.gluNurbsCallbackDataEXT )
def gluNurbsCallbackDataEXT( baseFunction,nurb, userData ):
    """Note the Python object for use as userData by the nurb"""
    return baseFunction(
        nurb, nurb.noteObject( userData )
    )

@_lazy( _simple.gluNurbsCurve )
def gluNurbsCurve( baseFunction, nurb, knots, control, type ):
    """Pythonic version of gluNurbsCurve

    Calculates knotCount, stride, and order automatically
    """
    knots = arrays.GLfloatArray.asArray( knots )
    knotCount = arrays.GLfloatArray.arraySize( knots )
    control = arrays.GLfloatArray.asArray( control )
    try:
        length,step = arrays.GLfloatArray.dimensions( control )
    except ValueError as err:
        raise error.GLUError( """Need a 2-dimensional control array""" )
    order = knotCount - length
    if _configflags.ERROR_CHECKING:
        checkOrder( order, knotCount, 'order of NURBS curve')
        checkKnots( knots, 'knots of NURBS curve')
    return baseFunction(
        nurb, knotCount, knots, step, control, order, type,
    )

@_lazy( _simple.gluNurbsSurface )
def gluNurbsSurface( baseFunction, nurb, sKnots, tKnots, control, type ):
    """Pythonic version of gluNurbsSurface

    Calculates knotCount, stride, and order automatically
    """
    sKnots = arrays.GLfloatArray.asArray( sKnots )
    sKnotCount = arrays.GLfloatArray.arraySize( sKnots )
    tKnots = arrays.GLfloatArray.asArray( tKnots )
    tKnotCount = arrays.GLfloatArray.arraySize( tKnots )
    control = arrays.GLfloatArray.asArray( control )

    try:
        length,width,step = arrays.GLfloatArray.dimensions( control )
    except ValueError as err:
        raise error.GLUError( """Need a 3-dimensional control array""" )
    sOrder = sKnotCount - length
    tOrder = tKnotCount - width
    sStride = width*step
    tStride = step
    if _configflags.ERROR_CHECKING:
        checkOrder( sOrder, sKnotCount, 'sOrder of NURBS surface')
        checkOrder( tOrder, tKnotCount, 'tOrder of NURBS surface')
        checkKnots( sKnots, 'sKnots of NURBS surface')
        checkKnots( tKnots, 'tKnots of NURBS surface')
    if not (sKnotCount-sOrder)*(tKnotCount-tOrder) == length*width:
        raise error.GLUError(
            """Invalid NURB structure""",
            nurb, sKnotCount, sKnots, tKnotCount, tKnots,
            sStride, tStride, control,
            sOrder,tOrder,
            type
        )

    result = baseFunction(
        nurb, sKnotCount, sKnots, tKnotCount, tKnots,
        sStride, tStride, control,
        sOrder,tOrder,
        type
    )
    return result

@_lazy( _simple.gluPwlCurve )
def gluPwlCurve( baseFunction, nurb, data, type ):
    """gluPwlCurve -- piece-wise linear curve within GLU context

    data -- the data-array
    type -- determines number of elements/data-point
    """
    data = arrays.GLfloatArray.asArray( data )
    if type == _simple.GLU_MAP1_TRIM_2:
        divisor = 2
    elif type == _simple.GLU_MAP_TRIM_3:
        divisor = 3
    else:
        raise ValueError( """Unrecognised type constant: %s"""%(type))
    size = arrays.GLfloatArray.arraySize( data )
    size = int(size//divisor)
    return baseFunction( nurb, size, data, divisor, type )



"""Late-bound base-class (with acceleration)"""
from OpenGL import acceleratesupport
LateBind = Curry = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.latebind import LateBind, Curry
    except ImportError as err:
        pass
if LateBind is None:
    class LateBind(object):
        """Provides a __call__ which dispatches to self._finalCall

        When called without self._finalCall() makes a call to
        self.finalise() and then calls self._finalCall()
        """
        _finalCall = None
        def setFinalCall( self, finalCall ):
            """Set our finalCall to the callable object given"""
            self._finalCall = finalCall
        def getFinalCall( self ):
            """Retrieve and/or bind and retrieve final call"""
            if not self._finalCall:
                self._finalCall = self.finalise()
            return self._finalCall


        def finalise( self ):
            """Finalise our target to our final callable object

            return final callable
            """

        def __call__( self, *args, **named ):
            """Call self._finalCall, calling finalise() first if not already called

            There's actually *no* reason to unpack and repack the arguments,
            but unfortunately I don't know of a Cython syntax to specify
            that.
            """
            try:
                return self._finalCall( *args, **named )
            except (TypeError,AttributeError) as err:
                if self._finalCall is None:
                    self._finalCall = self.finalise()
                return self._finalCall( *args, **named )
if Curry is None:
    class Curry(object):
        """Provides a simple Curry which can bind (only) the first element

        This is used by lazywrapper, which explains the weird naming
        of the two attributes...
        """
        wrapperFunction = None
        baseFunction = None
        def __init__( self, wrapperFunction, baseFunction ):
            """Stores self.wrapperFunction and self.baseFunction"""
            self.baseFunction = baseFunction
            self.wrapperFunction = wrapperFunction
        def __call__( self, *args, **named ):
            """returns self.wrapperFunction( self.baseFunction, *args, **named )"""
            return self.wrapperFunction( self.baseFunction, *args, **named )



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIS_texture_filter4'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIS_texture_filter4',error_checker=_errors._error_checker)
GL_FILTER4_SGIS=_C('GL_FILTER4_SGIS',0x8146)
GL_TEXTURE_FILTER4_SIZE_SGIS=_C('GL_TEXTURE_FILTER4_SIZE_SGIS',0x8147)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetTexFilterFuncSGIS(target,filter,weights):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLsizei,arrays.GLfloatArray)
def glTexFilterFuncSGIS(target,filter,n,weights):pass



'''OpenGL extension AMD.interleaved_elements

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.interleaved_elements to provide a more 
Python-friendly API

Overview (from the spec)
	
	    The glDrawElements function and its variants (instanced and indirect,
	for example) allow OpenGL to draw indexed arrays of vertices. Since its
	inception, OpenGL has supported unsigned bytes, unsigned shorts and
	unsigned integers as index types. However, all enabled vertex arrays may
	be represented by at most one shared index.
	
	    A common scenario in graphics rendering is that several faces share
	a vertex where, for each face some properties of a vertex (position and
	texture coordinates, for example) should be common but others must be
	unique (colors, normals, and so on). Consider a mesh of a cube with
	per-face normals, for example. There are 8 vertices and 6 normals, and 12
	triangles (where each face of the cube is represented as two triangles).
	To render this cube, we must compute the 24 unique permutations of
	position and normal and build a new element list to index into it. In
	fact, any advantage of indexed draw is lost here as the number of required
	permutations is equal to the final vertex count required to draw the
	object.
	
	    This extension allows OpenGL to process multi-component packed element
	data. The maximum size of a vertex's index data is not increased, but the
	facility to store 2 16-bit or 2 or 4 8-bit indices per vertex is introduced.
	Each vertex attribute is given a swizzle property to allow its index to
	be sourced from one of up to 4 channels of index data. This effectively
	allows an application to supply multiple interleaved streams of index data
	to OpenGL. Each vertex attribute is given a 'channel selector' to select
	one of the up to 4 channels of vertex index information presented to
	OpenGL. This enables the use-case described above and many more.
	The swizzle parameter is also applied to vertex indices passed to shaders,
	and updates to the definition of base vertex parameters and primitive
	restart are applied.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/interleaved_elements.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.interleaved_elements import *
from OpenGL.raw.GL.AMD.interleaved_elements import _EXTENSION_NAME

def glInitInterleavedElementsAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_draw_range_elements'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_draw_range_elements',error_checker=_errors._error_checker)
GL_MAX_ELEMENTS_INDICES_EXT=_C('GL_MAX_ELEMENTS_INDICES_EXT',0x80E9)
GL_MAX_ELEMENTS_VERTICES_EXT=_C('GL_MAX_ELEMENTS_VERTICES_EXT',0x80E8)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p)
def glDrawRangeElementsEXT(mode,start,end,count,type,indices):pass
